@@ -1,4 +1,4 @@
-#!/usr/local/bin/php-8.0
+#!/usr/local/bin/php-8.1
 <?php
 /*
  +-----------------------------------------------------------------------+
@@ -60,7 +60,7 @@ if (strtolower($input) == 'y') {
     $adds = [];
     $dirs = ['bin','SQL','plugins','skins','program'];
 
-    if (is_dir(INSTALL_PATH . 'vendor') && !is_file("$target_dir/composer.json")) {
+    if (is_dir(INSTALL_PATH . 'vendor') && (!is_file("$target_dir/composer.json") || rcmail_install::vendor_dir_untouched($target_dir))) {
         $dirs[] = 'vendor';
     }
     if (file_exists("$target_dir/installer")) {
